import static java.lang.Math.*;


public class atomFunc{
    static vectorFunc v;
    
    public atomFunc()
    {
        v = new vectorFunc();
    }
    
    public static atom zeroforce(atom i)    // reset force accumulator
    {
        atom j = new atom(i);
        j.force = new vector(0,0);    
        return j;
    }
    
    public static atom restingTest(atom i)  // test if moving, return bool
    {
        atom j = new atom(i);
        if (i.vel.x >= -10 && i.vel.x <= 10 && i.vel.y >= -10 && i.vel.y <= 10)
        
        {
            j.rested = true;
            return j;
        }
        else
        {
            j.rested = false;
            return j;
        }
    }
    
    public static boolean getrest(atom i)   // return rested state
    {        
        return i.rested;
    }
    
    public static atom unrest(atom i)       // set atom unrested
    {
        atom j = new atom(i);
        j.rested = false;
        return j;
    }
    
    public static atom rested(atom i)       // set atom rested
    {
        atom j = new atom(i);
        j.rested = true;
        return j;
    }
    
    public static double distanceCenters(atom i, atom j)    // find distance between atoms (center to center)
    {
        return v.mag(v.sub(i.pos, j.pos));
    }
    
    public static double distanceCentersSQ(atom i, atom j)    // find distanceSQ between atoms (center to center)
    {
        return v.magSQ(v.sub(i.pos, j.pos));
    }
    
    public static double distanceSurfaces(atom i, atom j)    // find distance between atoms (from surface to surface)
    {
        return v.mag(v.sub(i.pos, j.pos)) - sizes(i, j);
    }
    
    public static double distanceSurfacesSQ(atom i, atom j)    // find distanceSQ between atoms (from surface to surface)
    {
        return v.magSQ(v.sub(i.pos, j.pos)) - sizes(i, j);
    }
    
    public static vector separate(atom i)        // return randomized separation force adjusted for mass
    {
        double rx = (double)Math.random()+0.5d;
        double ry = (double)Math.random()-0.5d;
        return v.scale(new vector(rx, ry), rx+ry*1000*i.mass);
    }
    
    public static atom addforce(atom i, vector f)    // accumulate force
    {
        atom j = new atom(i);
        j.force = v.add(j.force, f);
        return j;
    }
    
    public static atom subforce(atom i, vector f)    // negative accumulate force
    {
        atom j = new atom(i);
        j.force = v.sub(j.force, f);
        return j;
    }
    
    public static double sizes(atom i, atom j)    // return combined radius of two atoms
    {
        return i.radius+j.radius;
    }
                                           
    public static vector repulse(atom i, atom j, double restlen, double distcent)    // return self repulsion spring+damper forces
    {
        vector link = v.sub(i.pos, j.pos);
        double mult = restlen /  ((double)sqrt(distcent) );
        //double mult =  (distcent - restlen)*3000000;
        double comp = (i.mass*j.mass)/(i.mass+j.mass);
        
        vector spring = v.scale(v.sub(v.scale(link, mult), link), comp*250000); // dist=distance surface to surface. (how deep) K=5000
        vector damper = v.scale((v.sub(j.vel, i.vel)), comp*50);            // D=damping
        return v.add(spring, damper);
    }
    
    public static atom gravity(atom i, vector grav)        // apply gravity on atom
    {
        atom j = new atom(i);
        j.force = v.add(j.force, v.scale(grav, j.mass));
        return j;
    }
    
    public static atom integrate(atom i, double dt)        // integrate system
    {
        atom j = new atom(i);
        j.vel = v.add(j.vel, v.scale(v.scale(j.force, j.imass), dt));     // v = (f/m) *dt   or   v = a*dt  or  v = integrated v
        j.pos = v.add(j.pos, v.scale(j.vel, dt));                           // x += vel *dt
        
        return j;
    }
    
    public static atom hardcollision(atom i, vector boxSize, vector boxPos, double decay, double friction) // only for 0' and 90' angles
    {
        atom j = new atom(i);               // test for box collide
        
        if (j.pos.x-j.radius < boxPos.x)     //else if can miss corner collisions
        {
            j.pos = new vector(boxPos.x+j.radius, j.pos.y);                 // adjust position
            j.vel = new vector(j.vel.x * -decay, j.vel.y * friction);       // adjust velocity with friction and wall bounce factors
        }
        if (j.pos.x+j.radius > boxPos.x+boxSize.x)
        {
            j.pos = new vector(boxPos.x+boxSize.x-j.radius, j.pos.y);
            j.vel = new vector(j.vel.x * -decay, j.vel.y * friction);
        }
        if (j.pos.y-j.radius < boxPos.y)
        {
            j.pos = new vector(j.pos.x, boxPos.y+j.radius);
            j.vel = new vector(j.vel.x * friction, j.vel.y * -decay);
        }
        if (j.pos.y+j.radius > boxPos.y+boxSize.y)
        {
            j.pos = new vector(j.pos.x, boxPos.y+boxSize.y-j.radius);
            j.vel = new vector(j.vel.x * friction, j.vel.y * -decay);
        }
        return j;
    }
    
    public static atom terminalforces(atom i)        // limit maximum forces in atom accumulator
    {
        atom j = new atom(i);
        if (j.force.x > 8000000)
        {
            j.force = new vector(8000000,j.force.y);
        }
        if (j.force.x < -8000000)
        {
            j.force = new vector(-8000000,j.force.y);
        }
        if (j.force.y > 8000000)
        {
            j.force = new vector(j.force.x,8000000);
        }
        if (j.force.y < -8000000)
        {
            j.force = new vector(j.force.x,-8000000);
        }
        return j;
    }
    
    public static atom terminalvelocity(atom i)        // limit maximum velocity
    {
        atom j = new atom(i);
        if (j.vel.x > 10000)
        {
            j.vel = new vector(10000,j.vel.y);
        }
        if (j.vel.x < -10000)
        {
            j.vel = new vector(-10000,j.vel.y);
        }
        if (j.vel.y > 10000)
        {
            j.vel = new vector(j.vel.x,10000);
        }
        if (j.vel.y < -10000)
        {
            j.vel = new vector(j.vel.x,-10000);
        }
        return j;
    }
    
    public static atom velocitydrag(atom i)        // apply scaling factor to velocity, simulate drag
    {
        atom j = new atom(i);
        j.vel = v.scale(j.vel, 0.998d);
        return j;
    }
    
    public static double getmass(atom i)    // get mass
    {       
        return i.mass;
    }
    
    public static vector getpos(atom i)    // get pos
    {       
        return i.pos;
    }
    
    public static vector getvel(atom i)    // get vel
    {       
        return i.vel;
    }
    
    public static double getsize(atom i)    // get size
    {       
        return i.radius;
    }
    
    //spring
    
    public static atom linkWith(atom atomSrc, int atomNum, double length)
    {
        if (atomSrc.linkQty >= atomSrc.maxLink-1)
            return atomSrc;
        atom nuatom = new atom(atomSrc);
        nuatom.linkedTo[nuatom.linkQty] = atomNum;
        nuatom.breaklimit[nuatom.linkQty] = 30000;
        nuatom.rigidity[nuatom.linkQty] = 30;
        nuatom.lenght[nuatom.linkQty] = length;
        nuatom.linkQty++;
        return nuatom;
    }
    
    public static atom springForce(atom atomSrc, atom[] atomLst)
    {
        if (atomSrc.linkQty == 0)
            return atomSrc;
        atom nuatom = new atom(atomSrc);
        atom atomDest;
        for (int i=0; i<atomSrc.linkQty;i++)
        {
            atomDest = atomLst[atomSrc.linkedTo[i]];
            double distCenters = (double)sqrt(distanceCentersSQ(atomSrc, atomDest));  // returns SQdistance of atoms centers
            if (distCenters == 0)
            {
                distCenters = 0.1d;                     // case distance = 0, make it work. (could delete spring too or break)
            }
            vector link = v.sub(atomSrc.pos, atomDest.pos);                // get vector between i and j
            double mult =  (distCenters - atomSrc.lenght[i])*-25000;                         // get force multiple: -K*( dist(i,j) - spring.lenght)
            double comp = (atomSrc.mass*atomDest.mass)/(atomSrc.mass+atomDest.mass);      // get mass compensation
            
            vector spring = v.scale(v.sub(v.scale(link, mult), link), comp*0.7);      // calculate spring forces
            vector damper = v.scale((v.sub(atomDest.vel, atomSrc.vel)), comp*50);                   // D=damping  (semi broken) 
         
            nuatom = addforce(nuatom, v.add(spring, damper));
        }
        return nuatom;
    }
};
